/*****************************************************************************
*   Presentation: 300 - Using Event Notifications in SQL Server 2005-2008
*   FileName:  	  08 - Blocked_Process_Event_Notification_Monitoring.sql
*
*   Summary: Demonstrates how to automate notifications of blocked processes
*            using Event Notifications and the blocked process report.
*
*   Date: September 7, 2011 
*
*   SQL Server Versions:
*         2005, 2008, 2008 R2
*         
******************************************************************************
*   Copyright (C) 2011 Jonathan M. Kehayias, SQLskills.com
*   All rights reserved. 
*
*   For more scripts and sample code, check out 
*      http://sqlskills.com/blogs/jonathan
*
*   You may alter this code for your own *non-commercial* purposes. You may
*   republish altered code as long as you include this copyright and give 
*	due credit. 
*
*
*   THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
*   TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
*   PARTICULAR PURPOSE. 
*
******************************************************************************/

-- Change databases to new database
USE [EventNotificationsDB];
GO

-- Create a table to old the blocked process reports as collected
CREATE TABLE [BlockedProcessReports]
([rowid] int identity primary key,
 [eventtype] varchar(100),
 [posttime] datetime,
 [blocked_process_report] xml,
 [original_message_body] xml not null );

GO

--  Create the Activation Stored Procedure to Process the Queue
IF EXISTS (SELECT * FROM dbo.sysobjects 
   WHERE id = OBJECT_ID(N'[dbo].[ProcessBlockedProcessReports]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[ProcessBlockedProcessReports]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[ProcessBlockedProcessReports]
WITH EXECUTE AS OWNER
AS 

DECLARE @message_body nvarchar(MAX)  
DECLARE @message_sequence_number int
DECLARE @dialog uniqueidentifier
DECLARE @email_message nvarchar(MAX)

WHILE (1 = 1)
BEGIN
	BEGIN TRANSACTION

-- Receive the next available message FROM the queue
	
	WAITFOR (
		RECEIVE TOP(1) -- just handle one message at a time
			@message_body=message_body,
			@message_sequence_number=message_sequence_number
			FROM dbo.BlockedProcessReportQueue
	), TIMEOUT 1000  -- if the queue is empty for one second, give UPDATE AND GO away

-- If we didn't get anything, bail out
	IF (@@ROWCOUNT = 0)
		BEGIN
			ROLLBACK TRANSACTION
			BREAK
		END 
		
	DECLARE 
		@blocked_process_report xml,	-- blocked process report xml document
		@stmtstart int,					-- start offset for executing statement
		@stmtend int,					-- end offset for executing statement
		@sqlhandle_txt varchar(64),		-- used for string match back into xmldoc
		@sqlhandle varbinary(64),		-- used for querying sys.dm_exec_sql_text DMF
		@sqltext nvarchar(max)			-- actual executing statement text
		
	-- Pop the next message off the Queue
	SELECT @blocked_process_report=CAST(@message_body AS xml).query('/EVENT_INSTANCE/TextData/*')

	-- Unfortunately, a cursor is required here to update the <frame> nodes
	-- with the new <sqltext> node for the statement actually being executed
	-- from a stored procedure.
	DECLARE node_cursor CURSOR FOR
	SELECT 
		stmtstart, 
		stmtend,
		sqlhandle, 
		CAST('' as xml).value('xs:hexBinary
								(substring(sql:column("t.sqlhandle"), 
										sql:column("t.pos")))', 
								'varbinary(max)') AS sql_handle
	FROM
	(
		SELECT 
			ISNULL(x.value('@stmtstart[1]', 'int'), 1) AS stmtstart,
			ISNULL(x.value('@stmtend[1]', 'int'), -1) AS stmtend,
			x.value('@sqlhandle[1]', 'varchar(64)') AS sqlhandle, 
			CASE SUBSTRING(x.value('@sqlhandle[1]', 'varchar(64)'), 1, 2) 
				WHEN '0x' 
					THEN 3 
				ELSE 0 
			END AS pos
		FROM @blocked_process_report.nodes('//frame') AS n(x)
	) AS t

	OPEN node_cursor

	FETCH NEXT FROM node_cursor 
	INTO @stmtstart, @stmtend, @sqlhandle_txt, @sqlhandle

	WHILE @@FETCH_STATUS = 0
	BEGIN

		-- Grab the sqltext from cache using the @sqlhandle varbinary variable    
		SELECT @sqltext = SUBSTRING(text, (@stmtstart/2)+1, 
			((CASE @stmtend
			  WHEN -1 THEN DATALENGTH(text)
			 ELSE @stmtend
			 END - @stmtstart)/2) + 1)
		FROM sys.dm_exec_sql_text(@sqlhandle)
			
		-- Insert a <sqltext/> node into the message frame
		-- This uses the @sqlhandle_txt variable to make the string match for simplicity.	
		SET @blocked_process_report.modify('insert <sqltext/> 
			into (//frame[@sqlhandle=sql:variable("@sqlhandle_txt")])[1]')

		-- Add the text data from the @sqltext sql variable to the new sqltext node
		SET @blocked_process_report.modify('insert text {sql:variable("@sqltext")}
			into (//frame[@sqlhandle=sql:variable("@sqlhandle_txt")]/sqltext)[1]')    
	            
		FETCH NEXT FROM node_cursor 
		INTO @stmtstart, @stmtend, @sqlhandle_txt, @sqlhandle
	END 
	CLOSE node_cursor
	DEALLOCATE node_cursor

	INSERT INTO BlockedProcessReports (eventtype, posttime, blocked_process_report, original_message_body)
	SELECT 
		message_data.value('(/EVENT_INSTANCE/EventType)[1]', 'varchar(128)' ),
		message_data.value('(/EVENT_INSTANCE/PostTime)[1]', 'varchar(128)'),
		@blocked_process_report,
		message_data
	FROM
	(SELECT cast( @message_body as xml ) as message_data ) as sub
	
	SELECT @email_message = CONVERT(nvarchar(max), @blocked_process_report)
    EXEC msdb.dbo.sp_send_dbmail
             @profile_name = 'DBA Notification Account', -- your defined email profile 
             @recipients = 'jmkehayias@gmail.com', -- your email
             @subject = 'Demo: Blocked Process Notification',
             @body = @email_message;
		
--  Commit the transaction.  At any point before this, we could roll 
--  back - the received message would be back on the queue AND the response
--  wouldn't be sent.
	COMMIT TRANSACTION
END
GO

-- Sign the procedure with the certificate's private key
ADD SIGNATURE TO OBJECT::[ProcessBlockedProcessReports]
BY CERTIFICATE [DBMailCertificate] 
WITH PASSWORD = '$tr0ngp@$$w0rd';
GO

--  Create a service broker queue to hold the events
CREATE QUEUE [BlockedProcessReportQueue]
	WITH STATUS=ON, 
		 ACTIVATION 
			(PROCEDURE_NAME = [ProcessBlockedProcessReports],
			 MAX_QUEUE_READERS = 1,
			 EXECUTE AS OWNER) 
GO

--  Create a service broker service receive the events
CREATE SERVICE [BlockedProcessReportService]
    ON QUEUE [BlockedProcessReportQueue] ([http://schemas.microsoft.com/SQL/Notifications/PostEventNotification])
GO

CREATE ROUTE [BlockedProcessReportRoute]
    WITH SERVICE_NAME = 'BlockedProcessReportService',
    ADDRESS = 'LOCAL';
GO


CREATE EVENT NOTIFICATION [CaptureBlockedProcessReports]
    ON SERVER
    WITH FAN_IN
    FOR BLOCKED_PROCESS_REPORT
    TO SERVICE 'BlockedProcessReportService', 'current database';
GO

EXEC sp_configure 'show advanced options', 1
GO
RECONFIGURE
GO
EXEC sp_configure 'blocked process threshold', 5
GO
RECONFIGURE
GO
EXEC sp_configure 'show advanced options', 0
GO
RECONFIGURE
GO

/*
--  Cleanup objects
DROP EVENT NOTIFICATION CaptureBlockedProcessReports ON SERVER
DROP ROUTE BlockedProcessReportRoute
DROP SERVICE BlockedProcessReportService
DROP QUEUE BlockedProcessReportQueue 
*/
