/*****************************************************************************
*   Presentation: 300 - Using Event Notifications in SQL Server 2005-2008
*   FileName:  	  10 - Auditing Role Changes.sql
*
*   Summary: Demonstrates automating email notifications for role changes
*            using Event Notifications.
*
*   Date: September 7, 2011 
*
*   SQL Server Versions:
*         2005, 2008, 2008 R2
*         
******************************************************************************
*   Copyright (C) 2011 Jonathan M. Kehayias, SQLskills.com
*   All rights reserved. 
*
*   For more scripts and sample code, check out 
*      http://sqlskills.com/blogs/jonathan
*
*   You may alter this code for your own *non-commercial* purposes. You may
*   republish altered code as long as you include this copyright and give 
*	due credit. 
*
*
*   THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
*   TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
*   PARTICULAR PURPOSE. 
*
******************************************************************************/

-- Change databases to new database
USE [EventNotificationsDB];
GO

--  Create the Activation Stored Procedure to Process the Queue
IF EXISTS (SELECT * FROM dbo.sysobjects 
   WHERE id = OBJECT_ID(N'[dbo].[ProcessAddRoleEvents]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[ProcessAddRoleEvents]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[ProcessAddRoleEvents]
WITH EXECUTE AS OWNER
AS 

DECLARE @message_body xml  
DECLARE @email_message nvarchar(MAX)

WHILE (1 = 1)
BEGIN
	BEGIN TRANSACTION

-- Receive the next available message FROM the queue
	
	WAITFOR (
		RECEIVE TOP(1) -- just handle one message at a time
			@message_body=message_body
			FROM dbo.AddRoleEventQueue
	), TIMEOUT 1000  -- if the queue is empty for one second, give UPDATE AND GO away

-- If we didn't get anything, bail out
	IF (@@ROWCOUNT = 0)
		BEGIN
			ROLLBACK TRANSACTION
			BREAK
		END 


		DECLARE @EventType varchar(128)
		DECLARE @ObjectName varchar(128)
		DECLARE @RoleName varchar(128)
		DECLARE @LoginName varchar(128)
		DECLARE @ServerName varchar(128)
		DECLARE @PostTime varchar(128)
		DECLARE @DatabaseName varchar(128)

		SELECT @EventType = @message_body.value('(/EVENT_INSTANCE/EventType)[1]', 'varchar(128)'),
				@ObjectName = @message_body.value('(/EVENT_INSTANCE/ObjectName)[1]', 'varchar(128)'),
				@RoleName = @message_body.value('(/EVENT_INSTANCE/RoleName)[1]', 'varchar(128)'),
				@LoginName = @message_body.value('(/EVENT_INSTANCE/LoginName)[1]', 'varchar(128)'),
				@ServerName = @message_body.value('(/EVENT_INSTANCE/ServerName)[1]', 'varchar(128)'),
				@PostTime = CAST(@message_body.value('(/EVENT_INSTANCE/PostTime)[1]', 'datetime') as varchar),
				@DatabaseName = @message_body.value('(/EVENT_INSTANCE/DatabaseName)[1]', 'varchar(128)')


		IF (@EventType IN ('ADD_SERVER_ROLE_MEMBER', 'DROP_SERVER_ROLE_MEMBER'))
			SELECT @email_message = 'The server login ' + ISNULL(@ObjectName, 'UNKNOWN') +
			' was' + CASE 
						WHEN @EventType = 'ADD_SERVER_ROLE_MEMBER' 
								THEN ' added to'
							WHEN @EventType = 'DROP_SERVER_ROLE_MEMBER'
								THEN ' removed from'
							ELSE ' UNKNOWN'
						END + ' the ' +
			ISNULL(@RoleName, 'UNKNOWN') + ' server role by login ' +
			ISNULL(@LoginName, 'UNKNOWN') + ' on server ' +
			ISNULL(@ServerName, 'UNKNOWN') + ' at ' +
			ISNULL(@PostTime, 'UNKNOWN')

		ELSE IF (@EventType IN ('ADD_ROLE_MEMBER', 'DROP_ROLE_MEMBER'))
			SELECT @email_message = 'The database user ' + ISNULL(@ObjectName, 'UNKNOWN') +
			' was' + CASE 
							WHEN @EventType = 'ADD_ROLE_MEMBER' 
								THEN ' added to'
							WHEN @EventType = 'DROP_ROLE_MEMBER'
								THEN ' removed from'
							ELSE ' UNKNOWN'
					  END + ' the' +
			ISNULL(@RoleName, 'UNKNOWN') + ' role in Database ' +
			ISNULL(@DatabaseName, 'UNKNOWN') + ' by login ' +
			ISNULL(@LoginName, 'UNKNOWN') + ' on server ' +
			ISNULL(@ServerName, 'UNKNOWN') + ' at ' +
			ISNULL(@PostTime, 'UNKNOWN')


	-- Send email using Database Mail
    EXEC msdb.dbo.sp_send_dbmail
             @profile_name = 'DBA Notification Account', -- your defined email profile 
             @recipients = 'email@domain.com', -- your email
             @subject = 'Demo: Add Role Event Notification',
             @body = @email_message;
		
--  Commit the transaction.  At any point before this, we could roll 
--  back - the received message would be back on the queue AND the response
--  wouldn't be sent.
	COMMIT TRANSACTION
END
GO

-- Sign the procedure with the certificate's private key
ADD SIGNATURE TO OBJECT::[ProcessAddRoleEvents]
BY CERTIFICATE [DBMailCertificate] 
WITH PASSWORD = '$tr0ngp@$$w0rd';
GO

--  Create a service broker queue to hold the events
CREATE QUEUE [AddRoleEventQueue]
	WITH STATUS=ON, 
		 ACTIVATION 
			(PROCEDURE_NAME = [ProcessAddRoleEvents],
			 MAX_QUEUE_READERS = 1,
			 EXECUTE AS OWNER) 
GO

--  Create a service broker service receive the events
CREATE SERVICE [AddRoleEventService]
    ON QUEUE [AddRoleEventQueue] ([http://schemas.microsoft.com/SQL/Notifications/PostEventNotification])
GO

CREATE ROUTE [AddRoleEventRoute]
    WITH SERVICE_NAME = 'AddRoleEventService',
    ADDRESS = 'LOCAL';
GO


CREATE EVENT NOTIFICATION [CaptureAddRoleEvents]
    ON SERVER
    WITH FAN_IN
    FOR ADD_SERVER_ROLE_MEMBER, ADD_ROLE_MEMBER, DROP_SERVER_ROLE_MEMBER, DROP_ROLE_MEMBER
    TO SERVICE 'AddRoleEventService', 'current database';
GO


/*
--  Cleanup objects
DROP EVENT NOTIFICATION CaptureAddRoleEvents ON SERVER
DROP ROUTE AddRoleEventRoute
DROP SERVICE AddRoleEventService
DROP QUEUE AddRoleEventQueue 
*/
