/*****************************************************************************
*   Presentation: 300 - Using Event Notifications in SQL Server 2005-2008
*   FileName:  	  11 - Sort_Warnings_Event_Notification_Monitoring.sql
*
*   Summary: Demonstrates using Event Notifications to capture information
*            about Sort Warning events in the system, as well as the last
*            executed statement for the session firing the event.
*
*   Date: September 7, 2011 
*
*   SQL Server Versions:
*         2005, 2008, 2008 R2
*         
******************************************************************************
*   Copyright (C) 2011 Jonathan M. Kehayias, SQLskills.com
*   All rights reserved. 
*
*   For more scripts and sample code, check out 
*      http://sqlskills.com/blogs/jonathan
*
*   You may alter this code for your own *non-commercial* purposes. You may
*   republish altered code as long as you include this copyright and give 
*	due credit. 
*
*
*   THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
*   TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
*   PARTICULAR PURPOSE. 
*
******************************************************************************/

-- Change databases to new database
USE [EventNotificationsDB];
GO

-- Create a table to old the blocked process reports as collected
CREATE TABLE [SortWarningsReports]
([rowid] int identity primary key,
 [eventtype] varchar(100),
 [posttime] datetime,
 [original_message_body] xml not null,
 [sqltext] nvarchar(max) );

GO

--  Create the Activation Stored Procedure to Process the Queue
IF EXISTS (SELECT * FROM dbo.sysobjects 
   WHERE id = OBJECT_ID(N'[dbo].[ProcessSortWarningsReports]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[ProcessSortWarningsReports]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[ProcessSortWarningsReports]
WITH EXECUTE AS OWNER
AS 

DECLARE @message_body xml  
DECLARE @message_sequence_number int
DECLARE @dialog uniqueidentifier
DECLARE @email_message nvarchar(MAX)

WHILE (1 = 1)
BEGIN
	BEGIN TRANSACTION

-- Receive the next available message FROM the queue
	
	WAITFOR (
		RECEIVE TOP(1) -- just handle one message at a time
			@message_body=message_body,
			@message_sequence_number=message_sequence_number
			FROM dbo.SortWarningsReportQueue
	), TIMEOUT 1000  -- if the queue is empty for one second, give UPDATE AND GO away

-- If we didn't get anything, bail out
	IF (@@ROWCOUNT = 0)
		BEGIN
			ROLLBACK TRANSACTION
			BREAK
		END 

	DECLARE 
		@spid int,						-- SPID generating SORT WARNING
		@sqltext nvarchar(max)			-- actual executing statement text

	SELECT @spid = @message_body.value('(/EVENT_INSTANCE/SPID)[1]', 'int' )

	SELECT @sqltext = [text] 
	FROM sys.dm_exec_connections
	CROSS APPLY sys.dm_exec_sql_text(most_recent_sql_handle)
	WHERE session_id = @spid
		

	INSERT INTO SortWarningsReports (eventtype, posttime, original_message_body, sqltext)
	SELECT 
		@message_body.value('(/EVENT_INSTANCE/EventType)[1]', 'varchar(128)' ),
		@message_body.value('(/EVENT_INSTANCE/PostTime)[1]', 'varchar(128)'),
		@message_body,
		@sqltext
		
--  Commit the transaction.  At any point before this, we could roll 
--  back - the received message would be back on the queue AND the response
--  wouldn't be sent.
	COMMIT TRANSACTION
END
GO

-- Sign the procedure with the certificate's private key
ADD SIGNATURE TO OBJECT::[ProcessSortWarningsReports]
BY CERTIFICATE [DBMailCertificate] 
WITH PASSWORD = '$tr0ngp@$$w0rd';
GO

--  Create a service broker queue to hold the events
CREATE QUEUE [SortWarningsReportQueue]
	WITH STATUS=ON, 
		 ACTIVATION 
			(PROCEDURE_NAME = [ProcessSortWarningsReports],
			 MAX_QUEUE_READERS = 1,
			 EXECUTE AS OWNER) 
GO

--  Create a service broker service receive the events
CREATE SERVICE [SortWarningsReportService]
    ON QUEUE [SortWarningsReportQueue] ([http://schemas.microsoft.com/SQL/Notifications/PostEventNotification])
GO

CREATE ROUTE [SortWarningsReportRoute]
    WITH SERVICE_NAME = 'SortWarningsReportService',
    ADDRESS = 'LOCAL';
GO


CREATE EVENT NOTIFICATION [CaptureSortWarningsReports]
    ON SERVER
    WITH FAN_IN
    FOR SORT_WARNINGS
    TO SERVICE 'SortWarningsReportService', 'current database';
GO


/*
--  Cleanup objects
DROP EVENT NOTIFICATION CaptureSortWarningsReports ON SERVER
DROP ROUTE SortWarningsReportRoute
DROP SERVICE SortWarningsReportService
DROP QUEUE SortWarningsReportQueue 
*/
